@@ -1,9 +1,9 @@
 /*
- * "$Id: file.c,v 1.13.2.51 2004/10/23 19:57:15 mike Exp $"
+ * "$Id: file.c 1461 2005-11-03 21:11:16Z mike $"
  *
  *   Filename routines for HTMLDOC, a HTML document processing program.
  *
- *   Copyright 1997-2004 by Easy Software Products.
+ *   Copyright 1997-2005 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -37,6 +37,7 @@
  *   file_method()      - Return the method for a filename or URL.
  *   file_nolocal()     - Disable access to local files.
  *   file_proxy()       - Set the proxy host for all HTTP requests.
+ *   file_referer()     - Set the HTTP referer for remote accesses.
  *   file_target()      - Return the target of a link.
  *   file_temp()        - Create and open a temporary file.
  */
@@ -103,6 +104,8 @@ int	web_files = 0,			/* Number of temporary files */
 cache_t	*web_cache = NULL;		/* Cache array */
 int	no_local = 0;			/* Non-zero to disable local files */
 char	cookies[1024] = "";		/* HTTP cookies, if any */
+char	referer_url[HTTP_MAX_VALUE] = "";
+					/* HTTP referer, if any */
 
 
 /*
@@ -148,6 +151,8 @@ file_cleanup(void)
 {
   int		i;			/* Looping var */
   char		filename[1024];		/* Temporary file */
+  struct stat	fileinfo;		/* File information */
+  size_t	remotebytes;		/* Size of remote data */
   const char	*tmpdir;		/* Temporary directory */
 #ifdef WIN32
   char		tmppath[1024];		/* Temporary directory */
@@ -172,12 +177,33 @@ file_cleanup(void)
     tmpdir = "/var/tmp";
 #endif /* WIN32 */
 
+ /*
+  * Report on the remote data bytes that were downloaded...
+  */
+
+  debug = getenv("HTMLDOC_DEBUG");
+
+  if (debug &&
+      (strstr(debug, "all") != NULL || strstr(debug, "remotebytes") != NULL))
+  {
+    for (i = 0, remotebytes = 0; i < web_files; i ++)
+      if (web_cache[i].url)
+      {
+	snprintf(filename, sizeof(filename), TEMPLATE, tmpdir,
+        	 (long)getpid(), i + 1);
+        if (!stat(filename, &fileinfo))
+	  remotebytes += fileinfo.st_size;
+      }
+
+    progress_error(HD_ERROR_NONE, "REMOTEBYTES: %ld", (long)remotebytes);
+  }
+
  /*
   * Check to see if we want to leave the temporary files around for
   * debugging...
   */
 
-  if ((debug = getenv("HTMLDOC_DEBUG")) != NULL &&
+  if (debug &&
       (strstr(debug, "all") != NULL || strstr(debug, "tempfiles") != NULL))
   {
    /*
@@ -479,6 +505,7 @@ file_find_check(const char *filename)	/* I - File or URL */
       httpSetField(http, HTTP_FIELD_HOST, hostname);
       httpSetField(http, HTTP_FIELD_USER_AGENT, "HTMLDOC v" SVERSION);
       httpSetField(http, HTTP_FIELD_CONNECTION, "Keep-Alive");
+      httpSetField(http, HTTP_FIELD_REFERER, referer_url);
 
       if (username[0])
       {
@@ -941,6 +968,20 @@ file_proxy(const char *url)	/* I - URL of proxy server */
 }
 
 
+/*
+ * 'file_referer()' - Set the HTTP referer for remote accesses.
+ */
+
+void
+file_referer(const char *referer)	/* I - Referer URL */
+{
+  if (referer)
+    strlcpy(referer_url, referer, sizeof(referer_url));
+  else
+    referer_url[0] = '\0';
+}
+
+
 /*
  * 'file_target()' - Return the target of a link.
  */
@@ -1048,5 +1089,5 @@ file_temp(char *name,			/* O - Filename */
 
 
 /*
- * End of "$Id: file.c,v 1.13.2.51 2004/10/23 19:57:15 mike Exp $".
+ * End of "$Id: file.c 1461 2005-11-03 21:11:16Z mike $".
  */
