@@ -1,9 +1,9 @@
 /*
- * "$Id: html.cxx,v 1.17.2.42 2004/10/23 06:16:41 mike Exp $"
+ * "$Id: html.cxx 1439 2005-10-28 20:32:59Z mike $"
  *
  *   HTML exporting functions for HTMLDOC, a HTML document processing program.
  *
- *   Copyright 1997-2004 by Easy Software Products.
+ *   Copyright 1997-2005 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -240,7 +240,12 @@ write_header(FILE   **out,	/* IO - Output file */
 		{
 		  "monospace",
 		  "serif",
-		  "sans-serif"
+		  "sans-serif",
+		  "monospace",
+		  "serif",
+		  "sans-serif",
+		  "symbol",
+		  "dingbats"
 		};
 
 
@@ -1108,5 +1113,5 @@ update_links(tree_t *t,		/* I - Document tree */
 
 
 /*
- * End of "$Id: html.cxx,v 1.17.2.42 2004/10/23 06:16:41 mike Exp $".
+ * End of "$Id: html.cxx 1439 2005-10-28 20:32:59Z mike $".
  */
