@@ -1,9 +1,9 @@
 /*
- * "$Id: htmldoc.cxx,v 1.36.2.77 2004/10/06 19:45:40 mike Exp $"
+ * "$Id: htmldoc.cxx 1461 2005-11-03 21:11:16Z mike $"
  *
  *   Main entry for HTMLDOC, a HTML document processing program.
  *
- *   Copyright 1997-2004 by Easy Software Products.
+ *   Copyright 1997-2005 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -27,6 +27,7 @@
  *   prefs_getrc()     - Get the rc file for preferences...
  *   prefs_load()      - Load HTMLDOC preferences...
  *   prefs_save()      - Save HTMLDOC preferences...
+ *   prefs_set_paths() - Set HTMLDOC data/help paths...
  *   compare_strings() - Compare two command-line strings.
  *   load_book()       - Load a book file...
  *   parse_options()   - Parse options from a book file...
@@ -55,6 +56,7 @@
 #else
 #  include <signal.h>
 #  include <unistd.h>
+#  include <sys/time.h>
 #endif // WIN32
 
 #ifdef __EMX__
@@ -76,8 +78,9 @@ typedef int (*exportfunc_t)(tree_t *, tree_t *);
  */
 
 static int	compare_strings(const char *s, const char *t, int tmin);
+static double	get_seconds(void);
 static int	load_book(const char *filename, tree_t **document,
-		          exportfunc_t *exportfunc);
+		          exportfunc_t *exportfunc, int set_nolocal = 0);
 static void	parse_options(const char *line, exportfunc_t *exportfunc);
 static int	read_file(const char *filename, tree_t **document,
 		          const char *path);
@@ -107,8 +110,14 @@ main(int  argc,				/* I - Number of command-line arguments */
   float		fontsize,		/* Base font size */
 		fontspacing;		/* Base font spacing */
   int		num_files;		/* Number of files provided on the command-line */
+  double	start_time,		/* Start time */
+		load_time,		/* Load time */
+		end_time;		/* End time */
+  const char	*debug;			/* HTMLDOC_DEBUG environment variable */
 
 
+  start_time = get_seconds();
+
 #ifdef __APPLE__
  /*
   * OSX passes an extra command-line option when run from the Finder.
@@ -139,6 +148,12 @@ main(int  argc,				/* I - Number of command-line arguments */
   signal(SIGTERM, term_handler);
 #endif // WIN32
 
+ /*
+  * Set the location of data and help files...
+  */
+
+  prefs_set_paths();
+
  /*
   * Check if we are being executed as a CGI program...
   */
@@ -174,10 +189,14 @@ main(int  argc,				/* I - Number of command-line arguments */
     PDFFirstPage  = PDF_PAGE_1;
 
     file_cookies(getenv("HTTP_COOKIE"));
-    file_nolocal();
+    file_referer(getenv("HTTP_REFERER"));
 
     progress_error(HD_ERROR_NONE, "INFO: HTMLDOC " SVERSION " starting in CGI mode.");
-    progress_error(HD_ERROR_NONE, "INFO: TMPDIR is \"%s\"\n", getenv("TMPDIR"));
+#ifdef WIN32
+    progress_error(HD_ERROR_NONE, "INFO: TEMP is \"%s\"", getenv("TEMP"));
+#else
+    progress_error(HD_ERROR_NONE, "INFO: TMPDIR is \"%s\"", getenv("TMPDIR"));
+#endif // WIN32
 
     argc = 1;
 
@@ -205,7 +224,9 @@ main(int  argc,				/* I - Number of command-line arguments */
       strlcpy(bookfile, ".book", sizeof(bookfile));
 
     if (!access(bookfile, 0))
-      load_book(bookfile, &document, &exportfunc);
+      load_book(bookfile, &document, &exportfunc, 1);
+    else
+      file_nolocal();
   }
   else
   {
@@ -263,15 +284,19 @@ main(int  argc,				/* I - Number of command-line arguments */
       i ++;
       if (i < argc)
       {
-        if (strcasecmp(argv[i], "courier") == 0 ||
-	    strcasecmp(argv[i], "monospace") == 0)
+        if (!strcasecmp(argv[i], "monospace"))
+	  _htmlBodyFont = TYPE_MONOSPACE;
+        else if (!strcasecmp(argv[i], "serif"))
+	  _htmlBodyFont = TYPE_SERIF;
+        else if (!strcasecmp(argv[i], "sans-serif") ||
+	         !strcasecmp(argv[i], "sans"))
+	  _htmlBodyFont = TYPE_SANS_SERIF;
+        else if (!strcasecmp(argv[i], "courier"))
 	  _htmlBodyFont = TYPE_COURIER;
-        else if (strcasecmp(argv[i], "times") == 0 ||
-	         strcasecmp(argv[i], "serif") == 0)
+        else if (!strcasecmp(argv[i], "times"))
 	  _htmlBodyFont = TYPE_TIMES;
-        else if (strcasecmp(argv[i], "helvetica") == 0 ||
-	         strcasecmp(argv[i], "arial") == 0 ||
-		 strcasecmp(argv[i], "sans-serif") == 0)
+        else if (!strcasecmp(argv[i], "helvetica") ||
+	         !strcasecmp(argv[i], "arial"))
 	  _htmlBodyFont = TYPE_HELVETICA;
       }
       else
@@ -511,7 +536,7 @@ main(int  argc,				/* I - Number of command-line arguments */
       OutputColor    = 0;
       _htmlGrayscale = 1;
     }
-    else if (compare_strings(argv[i], "--header", 7) == 0)
+    else if (!strcmp(argv[i], "--header"))
     {
       i ++;
       if (i < argc)
@@ -519,72 +544,145 @@ main(int  argc,				/* I - Number of command-line arguments */
       else
         usage(argv[i - 1]);
     }
-    else if (compare_strings(argv[i], "--headfootfont", 11) == 0)
+    else if (!strcmp(argv[i], "--header1"))
+    {
+      i ++;
+      if (i < argc)
+        get_format(argv[i], Header1);
+      else
+        usage(argv[i - 1]);
+    }
+    else if (!compare_strings(argv[i], "--headfootfont", 11))
     {
       i ++;
       if (i < argc)
       {
-        if (strcasecmp(argv[i], "courier") == 0)
+        if (!strcasecmp(argv[i], "courier"))
 	{
 	  HeadFootType  = TYPE_COURIER;
 	  HeadFootStyle = STYLE_NORMAL;
 	}
-        else if (strcasecmp(argv[i], "courier-bold") == 0)
+        else if (!strcasecmp(argv[i], "courier-bold"))
 	{
 	  HeadFootType  = TYPE_COURIER;
 	  HeadFootStyle = STYLE_BOLD;
 	}
-        else if (strcasecmp(argv[i], "courier-oblique") == 0)
+        else if (!strcasecmp(argv[i], "courier-oblique"))
 	{
 	  HeadFootType  = TYPE_COURIER;
 	  HeadFootStyle = STYLE_ITALIC;
 	}
-        else if (strcasecmp(argv[i], "courier-boldoblique") == 0)
+        else if (!strcasecmp(argv[i], "courier-boldoblique"))
 	{
 	  HeadFootType  = TYPE_COURIER;
 	  HeadFootStyle = STYLE_BOLD_ITALIC;
 	}
-        else if (strcasecmp(argv[i], "times") == 0 ||
-	         strcasecmp(argv[i], "times-roman") == 0)
+        else if (!strcasecmp(argv[i], "times") ||
+	         !strcasecmp(argv[i], "times-roman"))
 	{
 	  HeadFootType  = TYPE_TIMES;
 	  HeadFootStyle = STYLE_NORMAL;
 	}
-        else if (strcasecmp(argv[i], "times-bold") == 0)
+        else if (!strcasecmp(argv[i], "times-bold"))
 	{
 	  HeadFootType  = TYPE_TIMES;
 	  HeadFootStyle = STYLE_BOLD;
 	}
-        else if (strcasecmp(argv[i], "times-italic") == 0)
+        else if (!strcasecmp(argv[i], "times-italic"))
 	{
 	  HeadFootType  = TYPE_TIMES;
 	  HeadFootStyle = STYLE_ITALIC;
 	}
-        else if (strcasecmp(argv[i], "times-bolditalic") == 0)
+        else if (!strcasecmp(argv[i], "times-bolditalic"))
 	{
 	  HeadFootType  = TYPE_TIMES;
 	  HeadFootStyle = STYLE_BOLD_ITALIC;
 	}
-        else if (strcasecmp(argv[i], "helvetica") == 0)
+        else if (!strcasecmp(argv[i], "helvetica"))
 	{
 	  HeadFootType  = TYPE_HELVETICA;
 	  HeadFootStyle = STYLE_NORMAL;
 	}
-        else if (strcasecmp(argv[i], "helvetica-bold") == 0)
+        else if (!strcasecmp(argv[i], "helvetica-bold"))
 	{
 	  HeadFootType  = TYPE_HELVETICA;
 	  HeadFootStyle = STYLE_BOLD;
 	}
-        else if (strcasecmp(argv[i], "helvetica-oblique") == 0)
+        else if (!strcasecmp(argv[i], "helvetica-oblique"))
 	{
 	  HeadFootType  = TYPE_HELVETICA;
 	  HeadFootStyle = STYLE_ITALIC;
 	}
-        else if (strcasecmp(argv[i], "helvetica-boldoblique") == 0)
+        else if (!strcasecmp(argv[i], "helvetica-boldoblique"))
 	{
 	  HeadFootType  = TYPE_HELVETICA;
 	  HeadFootStyle = STYLE_BOLD_ITALIC;
 	}
+        else if (!strcasecmp(argv[i], "monospace"))
+	{
+	  HeadFootType  = TYPE_MONOSPACE;
+	  HeadFootStyle = STYLE_NORMAL;
+	}
+        else if (!strcasecmp(argv[i], "monospace-bold"))
+	{
+	  HeadFootType  = TYPE_MONOSPACE;
+	  HeadFootStyle = STYLE_BOLD;
+	}
+        else if (!strcasecmp(argv[i], "monospace-oblique"))
+	{
+	  HeadFootType  = TYPE_MONOSPACE;
+	  HeadFootStyle = STYLE_ITALIC;
+	}
+        else if (!strcasecmp(argv[i], "monospace-boldoblique"))
+	{
+	  HeadFootType  = TYPE_MONOSPACE;
+	  HeadFootStyle = STYLE_BOLD_ITALIC;
+	}
+        else if (!strcasecmp(argv[i], "serif") ||
+	         !strcasecmp(argv[i], "serif-roman"))
+	{
+	  HeadFootType  = TYPE_SERIF;
+	  HeadFootStyle = STYLE_NORMAL;
+	}
+        else if (!strcasecmp(argv[i], "serif-bold"))
+	{
+	  HeadFootType  = TYPE_SERIF;
+	  HeadFootStyle = STYLE_BOLD;
+	}
+        else if (!strcasecmp(argv[i], "serif-italic"))
+	{
+	  HeadFootType  = TYPE_SERIF;
+	  HeadFootStyle = STYLE_ITALIC;
+	}
+        else if (!strcasecmp(argv[i], "serif-bolditalic"))
+	{
+	  HeadFootType  = TYPE_SERIF;
+	  HeadFootStyle = STYLE_BOLD_ITALIC;
+	}
+        else if (!strcasecmp(argv[i], "sans-serif") ||
+	         !strcasecmp(argv[i], "sans"))
+	{
+	  HeadFootType  = TYPE_SANS_SERIF;
+	  HeadFootStyle = STYLE_NORMAL;
+	}
+        else if (!strcasecmp(argv[i], "sans-serif-bold") ||
+	         !strcasecmp(argv[i], "sans-bold"))
+	{
+	  HeadFootType  = TYPE_SANS_SERIF;
+	  HeadFootStyle = STYLE_BOLD;
+	}
+        else if (!strcasecmp(argv[i], "sans-serif-oblique") ||
+	         !strcasecmp(argv[i], "sans-oblique"))
+	{
+	  HeadFootType  = TYPE_SANS_SERIF;
+	  HeadFootStyle = STYLE_ITALIC;
+	}
+        else if (!strcasecmp(argv[i], "sans-serif-boldoblique") ||
+	         !strcasecmp(argv[i], "sans-boldoblique"))
+	{
+	  HeadFootType  = TYPE_SANS_SERIF;
+	  HeadFootStyle = STYLE_BOLD_ITALIC;
+	}
       }
       else
         usage(argv[i - 1]);
@@ -604,21 +702,25 @@ main(int  argc,				/* I - Number of command-line arguments */
       else
         usage(argv[i - 1]);
     }
-    else if (compare_strings(argv[i], "--headingfont", 7) == 0)
+    else if (!compare_strings(argv[i], "--headingfont", 7))
     {
       i ++;
       if (i < argc)
       {
-        if (strcasecmp(argv[i], "courier") == 0 ||
-	    strcasecmp(argv[i], "monospace") == 0)
+        if (!strcasecmp(argv[i], "courier"))
 	  _htmlHeadingFont = TYPE_COURIER;
-        else if (strcasecmp(argv[i], "times") == 0 ||
-	         strcasecmp(argv[i], "serif") == 0)
+        else if (!strcasecmp(argv[i], "times"))
 	  _htmlHeadingFont = TYPE_TIMES;
-        else if (strcasecmp(argv[i], "helvetica") == 0 ||
-	         strcasecmp(argv[i], "arial") == 0 ||
-	         strcasecmp(argv[i], "sans-serif") == 0)
+        else if (!strcasecmp(argv[i], "helvetica") ||
+	         !strcasecmp(argv[i], "arial"))
 	  _htmlHeadingFont = TYPE_HELVETICA;
+        else if (!strcasecmp(argv[i], "monospace"))
+	  _htmlHeadingFont = TYPE_MONOSPACE;
+        else if (!strcasecmp(argv[i], "serif"))
+	  _htmlHeadingFont = TYPE_SERIF;
+        else if (!strcasecmp(argv[i], "sans-serif") ||
+	         !strcasecmp(argv[i], "sans"))
+	  _htmlHeadingFont = TYPE_SANS_SERIF;
       }
       else
         usage(argv[i - 1]);
@@ -725,6 +827,8 @@ main(int  argc,				/* I - Number of command-line arguments */
       file_nolocal();
     else if (compare_strings(argv[i], "--no-numbered", 6) == 0)
       TocNumbers = 0;
+    else if (compare_strings(argv[i], "--no-overflow", 6) == 0)
+      OverflowErrors = 0;
     else if (compare_strings(argv[i], "--no-pscommands", 6) == 0)
       PSCommands = 0;
     else if (compare_strings(argv[i], "--no-strict", 6) == 0)
@@ -796,6 +900,8 @@ main(int  argc,				/* I - Number of command-line arguments */
       else
         usage(argv[i - 1]);
     }
+    else if (compare_strings(argv[i], "--overflow", 4) == 0)
+      OverflowErrors = 1;
     else if (compare_strings(argv[i], "--owner-password", 4) == 0)
     {
       i ++;
@@ -893,7 +999,15 @@ main(int  argc,				/* I - Number of command-line arguments */
       PSCommands = 1;
     else if (compare_strings(argv[i], "--quiet", 3) == 0)
       Verbosity = -1;
-    else if (compare_strings(argv[i], "--right", 3) == 0)
+    else if (!compare_strings(argv[i], "--referer", 4))
+    {
+      i ++;
+      if (i < argc)
+        file_referer(argv[i]);
+      else
+        usage(argv[i - 1]);
+    }
+    else if (compare_strings(argv[i], "--right", 4) == 0)
     {
       i ++;
       if (i < argc)
@@ -1143,6 +1257,8 @@ main(int  argc,				/* I - Number of command-line arguments */
   // Fix links...
   htmlFixLinks(document, document);
 
+  load_time = get_seconds();
+
   // Show debug info...
   htmlDebugStats("Document Tree", document);
 
@@ -1163,6 +1279,22 @@ main(int  argc,				/* I - Number of command-line arguments */
 
   (*exportfunc)(document, toc);
 
+  end_time = get_seconds();
+
+ /*
+  * Report running time statistics as needed...
+  */
+
+  if ((debug = getenv("HTMLDOC_DEBUG")) != NULL &&
+      (strstr(debug, "all") != NULL || strstr(debug, "timing") != NULL))
+    progress_error(HD_ERROR_NONE, "TIMING: %.3f %.3f %.3f",
+                   load_time - start_time, end_time - load_time,
+		   end_time - start_time);
+
+ /*
+  * Cleanup...
+  */
+
   htmlDeleteTree(document);
   htmlDeleteTree(toc);
 
@@ -1231,78 +1363,8 @@ prefs_load(void)
   int	pos;			// Header/footer position
   char	line[2048];		// Line from RC file
   FILE	*fp;			// File pointer
-#ifdef WIN32			//// Do registry magic...
-  HKEY		key;		// Registry key
-  DWORD		size;		// Size of string
-  static char	data[1024];	// Data directory
-  static char	doc[1024];	// Documentation directory
-  static char	path[4096];	// PATH environment variable
-#endif // WIN32
 
 
-  //
-  // Get the installed directories...
-  //
-
-#ifdef WIN32
-  // Open the registry...
-  if (!RegOpenKeyEx(HKEY_LOCAL_MACHINE,
-                    "SOFTWARE\\Easy Software Products\\HTMLDOC", 0,
-                    KEY_READ, &key))
-  {
-    // Grab the installed directories...
-    size = sizeof(data);
-    if (!RegQueryValueEx(key, "data", NULL, NULL, (unsigned char *)data, &size))
-      _htmlData = data;
-
-#  ifdef HAVE_LIBFLTK
-    size = sizeof(doc);
-    if (!RegQueryValueEx(key, "doc", NULL, NULL, (unsigned char *)doc, &size))
-      GUI::help_dir = doc;
-#  endif // HAVE_LIBFLTK
-
-    RegCloseKey(key);
-  }
-
-  // See if the HTMLDOC program folder is in the system execution path...
-  if (!RegOpenKeyEx(HKEY_LOCAL_MACHINE,
-                    "SYSTEM\\CurrentControlSet\\Control\\Session Manager\\Environment",
-                    0, KEY_READ | KEY_WRITE, &key))
-  {
-    // Grab the current path...
-    size = sizeof(path);
-    if (!RegQueryValueEx(key, "Path", NULL, NULL, (unsigned char *)path, &size))
-      if (strstr(path, _htmlData) == NULL)
-      {
-        // The data directory is not in the path, so add it...
-        strlcat(path, ";", sizeof(path));
-        strlcat(path, _htmlData, sizeof(path));
-        RegSetValueEx(key, "Path", 0, REG_EXPAND_SZ, (unsigned char *)path, strlen(path) + 1);
-      }
-  }
-#endif // WIN32
-
-#if defined(__EMX__) && defined(HAVE_LIBFLTK)
-  // If being installed within XFree86 OS/2 Environment
-  // we can use those values which are overwritten by
-  // the according environment variables.
-  _htmlData = strdup(__XOS2RedirRoot("/XFree86/lib/X11/htmldoc"));
-  GUI::help_dir = strdup(__XOS2RedirRoot("/XFree86/lib/X11/htmldoc/doc"));
-#endif // __EMX__ && HAVE_LIBFLTK
-
-  //
-  // See if the installed directories have been overridden by
-  // environment variables...
-  //
-
-  if (getenv("HTMLDOC_DATA") != NULL)
-    _htmlData = getenv("HTMLDOC_DATA");
-
-#ifdef HAVE_LIBFLTK
-  if (getenv("HTMLDOC_HELP") != NULL)
-    GUI::help_dir = getenv("HTMLDOC_HELP");
-#endif // HAVE_LIBFLTK
-
   //
   // Read the preferences file...
   //
@@ -1553,6 +1615,91 @@ prefs_save(void)
 }
 
 
+/*
+ * 'prefs_set_paths()' - Set HTMLDOC data/help paths...
+ */
+
+void
+prefs_set_paths(void)
+{
+#ifdef WIN32			//// Do registry magic...
+  HKEY		key;		// Registry key
+  DWORD		size;		// Size of string
+  static char	data[1024];	// Data directory
+  static char	doc[1024];	// Documentation directory
+  static char	path[4096];	// PATH environment variable
+#endif // WIN32
+
+
+  //
+  // Get the installed directories...
+  //
+
+#ifdef WIN32
+  // Open the registry...
+  if (!RegOpenKeyEx(HKEY_LOCAL_MACHINE,
+                    "SOFTWARE\\Easy Software Products\\HTMLDOC", 0,
+                    KEY_READ, &key))
+  {
+    // Grab the installed directories...
+    size = sizeof(data);
+    if (!RegQueryValueEx(key, "data", NULL, NULL, (unsigned char *)data, &size))
+      _htmlData = data;
+    else
+      progress_error(HD_ERROR_FILE_NOT_FOUND, "Unable to read \"data\" value from registry!");
+
+#  ifdef HAVE_LIBFLTK
+    size = sizeof(doc);
+    if (!RegQueryValueEx(key, "doc", NULL, NULL, (unsigned char *)doc, &size))
+      GUI::help_dir = doc;
+#  endif // HAVE_LIBFLTK
+
+    RegCloseKey(key);
+  }
+  else
+    progress_error(HD_ERROR_FILE_NOT_FOUND, "Unable to read HTMLDOC installation from registry!");
+
+  // See if the HTMLDOC program folder is in the system execution path...
+  if (!RegOpenKeyEx(HKEY_LOCAL_MACHINE,
+                    "SYSTEM\\CurrentControlSet\\Control\\Session Manager\\Environment",
+                    0, KEY_READ | KEY_WRITE, &key))
+  {
+    // Grab the current path...
+    size = sizeof(path);
+    if (!RegQueryValueEx(key, "Path", NULL, NULL, (unsigned char *)path, &size))
+      if (strstr(path, _htmlData) == NULL)
+      {
+        // The data directory is not in the path, so add it...
+        strlcat(path, ";", sizeof(path));
+        strlcat(path, _htmlData, sizeof(path));
+        RegSetValueEx(key, "Path", 0, REG_EXPAND_SZ, (unsigned char *)path, strlen(path) + 1);
+      }
+  }
+#endif // WIN32
+
+#if defined(__EMX__) && defined(HAVE_LIBFLTK)
+  // If being installed within XFree86 OS/2 Environment
+  // we can use those values which are overwritten by
+  // the according environment variables.
+  _htmlData = strdup(__XOS2RedirRoot("/XFree86/lib/X11/htmldoc"));
+  GUI::help_dir = strdup(__XOS2RedirRoot("/XFree86/lib/X11/htmldoc/doc"));
+#endif // __EMX__ && HAVE_LIBFLTK
+
+  //
+  // See if the installed directories have been overridden by
+  // environment variables...
+  //
+
+  if (getenv("HTMLDOC_DATA") != NULL)
+    _htmlData = getenv("HTMLDOC_DATA");
+
+#ifdef HAVE_LIBFLTK
+  if (getenv("HTMLDOC_HELP") != NULL)
+    GUI::help_dir = getenv("HTMLDOC_HELP");
+#endif // HAVE_LIBFLTK
+}
+
+
 /*
  * 'compare_strings()' - Compare two command-line strings.
  */
@@ -1573,6 +1720,25 @@ compare_strings(const char *s,	/* I - Command-line string */
 }
 
 
+/*
+ * 'get_seconds()' - Get the current fractional time in seconds.
+ */
+
+static double				/* O - Number of seconds */
+get_seconds(void)
+{
+#ifdef WIN32
+  return (GetTickCount() * 0.001);
+#else
+  struct timeval	curtime;	/* Current time */
+
+  gettimeofday(&curtime, NULL);
+
+  return (curtime.tv_sec + curtime.tv_usec * 0.000001);
+#endif /* WIN32 */
+}
+
+
 //
 // 'load_book()' - Load a book file...
 //
@@ -1580,7 +1746,8 @@ compare_strings(const char *s,	/* I - Command-line string */
 static int				// O  - 1 = success, 0 = failure
 load_book(const char   *filename,	// I  - Book file
           tree_t       **document,	// IO - Document tree
-          exportfunc_t *exportfunc)	// O  - Export function
+          exportfunc_t *exportfunc,	// O  - Export function
+          int          set_nolocal)	// I  - Set file_nolocal() after lookup?
 {
   FILE		*fp;			// File to read from
   char		line[10240];		// Line from file
@@ -1598,7 +1765,12 @@ load_book(const char   *filename,	// I  - Book file
     strlcpy(path, Path, sizeof(path));
 
   // Open the file...
-  if ((local = file_find(Path, filename)) == NULL)
+  local = file_find(Path, filename);
+
+  if (set_nolocal)
+    file_nolocal();
+
+  if (!local)
     return (0);
 
   if ((fp = fopen(local, "rb")) == NULL)
@@ -1838,6 +2010,14 @@ parse_options(const char   *line,	// I - Options from book file
       Encryption = 0;
       continue;
     }
+    else if (strcmp(temp, "--strict") == 0)
+      StrictHTML = 1;
+    else if (strcmp(temp, "--no-strict") == 0)
+      StrictHTML = 0;
+    else if (strcmp(temp, "--overflow") == 0)
+      OverflowErrors = 1;
+    else if (strcmp(temp, "--no-overflow") == 0)
+      OverflowErrors = 0;
 
     if (*lineptr == '\"')
     {
@@ -1942,6 +2122,8 @@ parse_options(const char   *line,	// I - Options from book file
       PageBottom = get_measurement(temp2);
     else if (strcmp(temp, "--header") == 0)
       get_format(temp2, Header);
+    else if (strcmp(temp, "--header1") == 0)
+      get_format(temp2, Header1);
     else if (strcmp(temp, "--footer") == 0)
       get_format(temp2, Footer);
     else if (strcmp(temp, "--bodycolor") == 0)
@@ -1991,96 +2173,163 @@ parse_options(const char   *line,	// I - Options from book file
 
       htmlSetBaseSize(fontsize, fontspacing);
     }
-    else if (strcmp(temp, "--headingfont") == 0)
+    else if (!strcmp(temp, "--headingfont"))
     {
-      if (strcasecmp(temp2, "courier") == 0 ||
-          strcasecmp(temp2, "monospace") == 0)
-        _htmlHeadingFont = TYPE_COURIER;
-      else if (strcasecmp(temp2, "times") == 0 ||
-               strcasecmp(temp2, "serif") == 0)
-        _htmlHeadingFont = TYPE_TIMES;
-      else if (strcasecmp(temp2, "helvetica") == 0 ||
-               strcasecmp(temp2, "arial") == 0 ||
-               strcasecmp(temp2, "sans-serif") == 0)
-        _htmlHeadingFont = TYPE_HELVETICA;
+      if (!strcasecmp(temp2, "courier"))
+	_htmlHeadingFont = TYPE_COURIER;
+      else if (!strcasecmp(temp2, "times"))
+	_htmlHeadingFont = TYPE_TIMES;
+      else if (!strcasecmp(temp2, "helvetica") ||
+	       !strcasecmp(temp2, "arial"))
+	_htmlHeadingFont = TYPE_HELVETICA;
+      else if (!strcasecmp(temp2, "monospace"))
+	_htmlHeadingFont = TYPE_MONOSPACE;
+      else if (!strcasecmp(temp2, "serif"))
+	_htmlHeadingFont = TYPE_SERIF;
+      else if (!strcasecmp(temp2, "sans"))
+	_htmlHeadingFont = TYPE_SANS_SERIF;
     }
-    else if (strcmp(temp, "--bodyfont") == 0)
+    else if (!strcmp(temp, "--bodyfont"))
     {
-      if (strcasecmp(temp2, "courier") == 0 ||
-          strcasecmp(temp2, "monospace") == 0)
-        _htmlBodyFont = TYPE_COURIER;
-      else if (strcasecmp(temp2, "times") == 0 ||
-               strcasecmp(temp2, "serif") == 0)
-        _htmlBodyFont = TYPE_TIMES;
-      else if (strcasecmp(temp2, "helvetica") == 0 ||
-               strcasecmp(temp2, "arial") == 0 ||
-               strcasecmp(temp2, "sans-serif") == 0)
-        _htmlBodyFont = TYPE_HELVETICA;
+      if (!strcasecmp(temp2, "monospace"))
+	_htmlBodyFont = TYPE_MONOSPACE;
+      else if (!strcasecmp(temp2, "serif"))
+	_htmlBodyFont = TYPE_SERIF;
+      else if (!strcasecmp(temp2, "sans"))
+	_htmlBodyFont = TYPE_SANS_SERIF;
+      else if (!strcasecmp(temp2, "courier"))
+	_htmlBodyFont = TYPE_COURIER;
+      else if (!strcasecmp(temp2, "times"))
+	_htmlBodyFont = TYPE_TIMES;
+      else if (!strcasecmp(temp2, "helvetica") ||
+	       !strcasecmp(temp2, "arial"))
+	_htmlBodyFont = TYPE_HELVETICA;
     }
     else if (strcmp(temp, "--headfootsize") == 0)
       HeadFootSize = atof(temp2);
-    else if (strcmp(temp, "--headfootfont") == 0)
+    else if (!strcmp(temp, "--headfootfont"))
     {
-      if (strcasecmp(temp2, "courier") == 0)
+      if (!strcasecmp(temp2, "courier"))
       {
 	HeadFootType  = TYPE_COURIER;
 	HeadFootStyle = STYLE_NORMAL;
       }
-      else if (strcasecmp(temp2, "courier-bold") == 0)
+      else if (!strcasecmp(temp2, "courier-bold"))
       {
 	HeadFootType  = TYPE_COURIER;
 	HeadFootStyle = STYLE_BOLD;
       }
-      else if (strcasecmp(temp2, "courier-oblique") == 0)
+      else if (!strcasecmp(temp2, "courier-oblique"))
       {
 	HeadFootType  = TYPE_COURIER;
 	HeadFootStyle = STYLE_ITALIC;
       }
-      else if (strcasecmp(temp2, "courier-boldoblique") == 0)
+      else if (!strcasecmp(temp2, "courier-boldoblique"))
       {
 	HeadFootType  = TYPE_COURIER;
 	HeadFootStyle = STYLE_BOLD_ITALIC;
       }
-      else if (strcasecmp(temp2, "times") == 0 ||
-	         strcasecmp(temp2, "times-roman") == 0)
+      else if (!strcasecmp(temp2, "times") ||
+	       !strcasecmp(temp2, "times-roman"))
       {
 	HeadFootType  = TYPE_TIMES;
 	HeadFootStyle = STYLE_NORMAL;
       }
-      else if (strcasecmp(temp2, "times-bold") == 0)
+      else if (!strcasecmp(temp2, "times-bold"))
       {
 	HeadFootType  = TYPE_TIMES;
 	HeadFootStyle = STYLE_BOLD;
       }
-      else if (strcasecmp(temp2, "times-italic") == 0)
+      else if (!strcasecmp(temp2, "times-italic"))
       {
 	HeadFootType  = TYPE_TIMES;
 	HeadFootStyle = STYLE_ITALIC;
       }
-      else if (strcasecmp(temp2, "times-bolditalic") == 0)
+      else if (!strcasecmp(temp2, "times-bolditalic"))
       {
 	HeadFootType  = TYPE_TIMES;
 	HeadFootStyle = STYLE_BOLD_ITALIC;
       }
-      else if (strcasecmp(temp2, "helvetica") == 0)
+      else if (!strcasecmp(temp2, "helvetica"))
       {
 	HeadFootType  = TYPE_HELVETICA;
 	HeadFootStyle = STYLE_NORMAL;
       }
-      else if (strcasecmp(temp2, "helvetica-bold") == 0)
+      else if (!strcasecmp(temp2, "helvetica-bold"))
       {
 	HeadFootType  = TYPE_HELVETICA;
 	HeadFootStyle = STYLE_BOLD;
       }
-      else if (strcasecmp(temp2, "helvetica-oblique") == 0)
+      else if (!strcasecmp(temp2, "helvetica-oblique"))
       {
 	HeadFootType  = TYPE_HELVETICA;
 	HeadFootStyle = STYLE_ITALIC;
       }
-      else if (strcasecmp(temp2, "helvetica-boldoblique") == 0)
+      else if (!strcasecmp(temp2, "helvetica-boldoblique"))
       {
 	HeadFootType  = TYPE_HELVETICA;
-        HeadFootStyle = STYLE_BOLD_ITALIC;
+	HeadFootStyle = STYLE_BOLD_ITALIC;
+      }
+      else if (!strcasecmp(temp2, "monospace"))
+      {
+	HeadFootType  = TYPE_MONOSPACE;
+	HeadFootStyle = STYLE_NORMAL;
+      }
+      else if (!strcasecmp(temp2, "monospace-bold"))
+      {
+	HeadFootType  = TYPE_MONOSPACE;
+	HeadFootStyle = STYLE_BOLD;
+      }
+      else if (!strcasecmp(temp2, "monospace-oblique"))
+      {
+	HeadFootType  = TYPE_MONOSPACE;
+	HeadFootStyle = STYLE_ITALIC;
+      }
+      else if (!strcasecmp(temp2, "monospace-boldoblique"))
+      {
+	HeadFootType  = TYPE_MONOSPACE;
+	HeadFootStyle = STYLE_BOLD_ITALIC;
+      }
+      else if (!strcasecmp(temp2, "serif") ||
+	       !strcasecmp(temp2, "serif-roman"))
+      {
+	HeadFootType  = TYPE_SERIF;
+	HeadFootStyle = STYLE_NORMAL;
+      }
+      else if (!strcasecmp(temp2, "serif-bold"))
+      {
+	HeadFootType  = TYPE_SERIF;
+	HeadFootStyle = STYLE_BOLD;
+      }
+      else if (!strcasecmp(temp2, "serif-italic"))
+      {
+	HeadFootType  = TYPE_SERIF;
+	HeadFootStyle = STYLE_ITALIC;
+      }
+      else if (!strcasecmp(temp2, "serif-bolditalic"))
+      {
+	HeadFootType  = TYPE_SERIF;
+	HeadFootStyle = STYLE_BOLD_ITALIC;
+      }
+      else if (!strcasecmp(temp2, "sans"))
+      {
+	HeadFootType  = TYPE_SANS_SERIF;
+	HeadFootStyle = STYLE_NORMAL;
+      }
+      else if (!strcasecmp(temp2, "sans-bold"))
+      {
+	HeadFootType  = TYPE_SANS_SERIF;
+	HeadFootStyle = STYLE_BOLD;
+      }
+      else if (!strcasecmp(temp2, "sans-oblique"))
+      {
+	HeadFootType  = TYPE_SANS_SERIF;
+	HeadFootStyle = STYLE_ITALIC;
+      }
+      else if (!strcasecmp(temp2, "sans-boldoblique"))
+      {
+	HeadFootType  = TYPE_SANS_SERIF;
+	HeadFootStyle = STYLE_BOLD_ITALIC;
       }
     }
     else if (strcmp(temp, "--charset") == 0)
@@ -2299,7 +2548,7 @@ usage(const char *arg)			// I - Bad argument string
   if (CGIMode)
     puts("Content-Type: text/plain\r\n\r");
 
-  puts("HTMLDOC Version " SVERSION " Copyright 1997-2004 Easy Software Products, All Rights Reserved.");
+  puts("HTMLDOC Version " SVERSION " Copyright 1997-2005 Easy Software Products, All Rights Reserved.");
   puts("This software is governed by the GNU General Public License, Version 2, and");
   puts("is based in part on the work of the Independent JPEG Group.");
   puts("");
@@ -2322,7 +2571,7 @@ usage(const char *arg)			// I - Bad argument string
     puts("");
     puts("  --batch filename.book");
     puts("  --bodycolor color");
-    puts("  --bodyfont {courier,times,helvetica}");
+    puts("  --bodyfont {courier,helvetica,monospace,sans,serif,times}");
     puts("  --bodyimage filename.{bmp,gif,jpg,png}");
     puts("  --book");
     puts("  --bottom margin{in,cm,mm}");
@@ -2344,11 +2593,15 @@ usage(const char *arg)			// I - Bad argument string
     puts("  {--format, -t} {ps1,ps2,ps3,pdf11,pdf12,pdf13,pdf14,html,htmlsep}");
     puts("  --gray");
     puts("  --header fff");
+    puts("  --header1 fff");
     puts("  --headfootfont {courier{-bold,-oblique,-boldoblique},\n"
-	 "                  times{-roman,-bold,-italic,-bolditalic},\n"
-	 "                  helvetica{-bold,-oblique,-boldoblique}}");
+	 "                  helvetica{-bold,-oblique,-boldoblique},\n"
+	 "                  monospace{-bold,-oblique,-boldoblique},\n"
+	 "                  sans{-bold,-oblique,-boldoblique},\n"
+	 "                  serif{-bold,-italic,-bolditalic},\n"
+	 "                  times{-roman,-bold,-italic,-bolditalic}}\n");
     puts("  --headfootsize {6.0..24.0}");
-    puts("  --headingfont {courier,times,helvetica}");
+    puts("  --headingfont {courier,helvetica,monospace,sans,serif,times}");
     puts("  --help");
 #ifdef HAVE_LIBFLTK
     puts("  --helpdir directory");
@@ -2362,7 +2615,6 @@ usage(const char *arg)			// I - Bad argument string
     puts("  --links");
     puts("  --linkstyle {plain,underline}");
     puts("  --logoimage filename.{bmp,gif,jpg,png}");
-    puts("  --owner-password password");
     puts("  --no-compression");
     puts("  --no-duplex");
     puts("  --no-embedfonts");
@@ -2370,6 +2622,7 @@ usage(const char *arg)			// I - Bad argument string
     puts("  --no-links");
     puts("  --no-localfiles");
     puts("  --no-numbered");
+    puts("  --no-overflow");
     puts("  --no-pscommands");
     puts("  --no-strict");
     puts("  --no-title");
@@ -2378,6 +2631,8 @@ usage(const char *arg)			// I - Bad argument string
     puts("  --nup {1,2,4,6,9,16}");
     puts("  {--outdir, -d} dirname");
     puts("  {--outfile, -f} filename.{ps,pdf,html}");
+    puts("  --overflow");
+    puts("  --owner-password password");
     puts("  --pageduration {1.0..60.0}");
     puts("  --pageeffect {none,bi,bo,d,gd,gdr,gr,hb,hsi,hso,vb,vsi,vso,wd,wl,wr,wu}");
     puts("  --pagelayout {single,one,twoleft,tworight}");
@@ -2388,6 +2643,7 @@ usage(const char *arg)			// I - Bad argument string
     puts("  --proxy http://host:port");
     puts("  --pscommands");
     puts("  --quiet");
+    puts("  --referer url");
     puts("  --right margin{in,cm,mm}");
     puts("  --size {letter,a4,WxH{in,cm,mm},etc}");
     puts("  --strict");
@@ -2431,5 +2687,5 @@ usage(const char *arg)			// I - Bad argument string
 
 
 /*
- * End of "$Id: htmldoc.cxx,v 1.36.2.77 2004/10/06 19:45:40 mike Exp $".
+ * End of "$Id: htmldoc.cxx 1461 2005-11-03 21:11:16Z mike $".
  */
