@@ -1,9 +1,9 @@
 /*
- * "$Id: htmllib.cxx,v 1.41.2.79 2004/10/21 19:05:04 mike Exp $"
+ * "$Id: htmllib.cxx 1466 2005-11-18 01:37:50Z mike $"
  *
  *   HTML parsing routines for HTMLDOC, a HTML document processing program.
  *
- *   Copyright 1997-2004 by Easy Software Products.
+ *   Copyright 1997-2005 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -23,34 +23,35 @@
  *
  * Contents:
  *
- *   htmlReadFile()      - Read a file for HTML markup codes.
- *   write_file()        - Write a tree entry to a file...
- *   htmlWriteFile()     - Write an HTML markup tree to a file.
- *   htmlAddTree()       - Add a tree node to the parent.
- *   htmlDeleteTree()    - Free all memory associated with a tree...
- *   htmlInsertTree()    - Insert a tree node to the parent.
- *   htmlNewTree()       - Create a new tree node for the parent.
- *   htmlGetText()       - Get all text from the given tree.
- *   htmlGetMeta()       - Get document "meta" data...
- *   htmlGetStyle()      - Get a style value from a node's STYLE attribute.
- *   htmlGetVariable()   - Get a variable value from a markup entry.
- *   htmlSetVariable()   - Set a variable for a markup entry.
- *   htmlSetBaseSize()   - Set the font sizes and spacings...
- *   htmlSetCharSet()    - Set the character set for output.
- *   htmlSetTextColor()  - Set the default text color.
- *   compare_variables() - Compare two markup variables.
- *   compare_markups()   - Compare two markup strings...
- *   delete_node()       - Free all memory associated with a node...
- *   insert_space()      - Insert a whitespace character before the
- *                         specified node.
- *   parse_markup()      - Parse a markup string.
- *   parse_variable()    - Parse a markup variable string.
- *   compute_size()      - Compute the width and height of a tree entry.
- *   compute_color()     - Compute the red, green, blue color from the given
- *   get_alignment()     - Get horizontal & vertical alignment values.
- *   fix_filename()      - Fix a filename to be relative to the base directory.
- *   html_memory_used()  - Figure out the amount of memory that was used.
- *   htmlDebugStats()    - Display debug statistics for HTML tree memory use.
+ *   htmlReadFile()       - Read a file for HTML markup codes.
+ *   write_file()         - Write a tree entry to a file...
+ *   htmlWriteFile()      - Write an HTML markup tree to a file.
+ *   htmlAddTree()        - Add a tree node to the parent.
+ *   htmlDeleteTree()     - Free all memory associated with a tree...
+ *   htmlInsertTree()     - Insert a tree node to the parent.
+ *   htmlNewTree()        - Create a new tree node for the parent.
+ *   htmlGetText()        - Get all text from the given tree.
+ *   htmlGetMeta()        - Get document "meta" data...
+ *   htmlGetStyle()       - Get a style value from a node's STYLE attribute.
+ *   htmlGetVariable()    - Get a variable value from a markup entry.
+ *   htmlLoadFontWidths() - Load all of the font width files.
+ *   htmlSetVariable()    - Set a variable for a markup entry.
+ *   htmlSetBaseSize()    - Set the font sizes and spacings...
+ *   htmlSetCharSet()     - Set the character set for output.
+ *   htmlSetTextColor()   - Set the default text color.
+ *   compare_variables()  - Compare two markup variables.
+ *   compare_markups()    - Compare two markup strings...
+ *   delete_node()        - Free all memory associated with a node...
+ *   insert_space()       - Insert a whitespace character before the
+ *                          specified node.
+ *   parse_markup()       - Parse a markup string.
+ *   parse_variable()     - Parse a markup variable string.
+ *   compute_size()       - Compute the width and height of a tree entry.
+ *   compute_color()      - Compute the red, green, blue color from the given
+ *   get_alignment()      - Get horizontal & vertical alignment values.
+ *   fix_filename()       - Fix a filename to be relative to the base directory.
+ *   html_memory_used()   - Figure out the amount of memory that was used.
+ *   htmlDebugStats()     - Display debug statistics for HTML tree memory use.
  */
 
 /*
@@ -182,10 +183,14 @@ typeface_t	_htmlBodyFont = TYPE_TIMES,
 int		_htmlInitialized = 0;	/* Initialized glyphs yet? */
 char		_htmlCharSet[256] = "iso-8859-1";
 					/* Character set name */
-float		_htmlWidths[4][4][256];	/* Character widths of fonts */
+float		_htmlWidths[TYPE_MAX][STYLE_MAX][256];
+					/* Character widths of fonts */
 const char	*_htmlGlyphsAll[65536];	/* Character glyphs for Unicode */
 const char	*_htmlGlyphs[256];	/* Character glyphs for charset */
-const char	*_htmlFonts[4][4] =
+int		_htmlNumSorted = 0;	/* Number of sorted glyphs */
+const char	*_htmlSorted[256];	/* Sorted character glyphs for charset */
+uchar		_htmlSortedChars[256];	/* Sorted character indices */
+const char	*_htmlFonts[TYPE_MAX][STYLE_MAX] =
 		{
 		  {
 		    "Courier",
@@ -205,13 +210,48 @@ const char	*_htmlFonts[4][4] =
 		    "Helvetica-Oblique",
 		    "Helvetica-BoldOblique"
 		  },
+		  {
+		    "Monospace",
+		    "Monospace-Bold",
+		    "Monospace-Oblique",
+		    "Monospace-BoldOblique"
+		  },
+		  {
+		    "Serif-Roman",
+		    "Serif-Bold",
+		    "Serif-Oblique",
+		    "Serif-BoldOblique"
+		  },
+		  {
+		    "Sans",
+		    "Sans-Bold",
+		    "Sans-Oblique",
+		    "Sans-BoldOblique"
+		  },
 		  {
 		    "Symbol",
 		    "Symbol",
 		    "Symbol",
 		    "Symbol"
+		  },
+		  {
+		    "Dingbats",
+		    "Dingbats",
+		    "Dingbats",
+		    "Dingbats"
 		  }
 		};
+int		_htmlStandardFonts[TYPE_MAX] =
+		{
+		  1,	// Courier
+		  1,	// Times
+		  1,	// Helvetica
+		  0,	// Monospace
+		  0,	// Sans
+		  0,	// Serif
+		  1,	// Symbol
+		  0	// Dingbats
+		};
 
 
 /*
@@ -563,6 +603,21 @@ htmlReadFile(tree_t     *parent,	// I - Parent tree entry
 	    else if (temp->markup == MARKUP_TABLE || istable(temp->markup) ||
 	             temp->markup == MARKUP_EMBED)
 	    {
+	      if (temp->markup != MARKUP_TR)
+	      {
+	        // Strictly speaking, this is an error - TD/TH can only
+		// be found under TR, but web browsers automatically
+		// inject a TR...
+		progress_error(HD_ERROR_HTML_ERROR,
+		               "No TR element before %s element on line %d.",
+			       _htmlMarkups[t->markup], linenum);
+
+                parent = htmlAddTree(temp, MARKUP_TR, NULL);
+		prev   = NULL;
+		DEBUG_printf(("%str (inserted) under %s, line %d\n", indent,
+		              _htmlMarkups[temp->markup], linenum));
+	      }
+
 	      temp = NULL;
               break;
 	    }
@@ -574,7 +629,7 @@ htmlReadFile(tree_t     *parent,	// I - Parent tree entry
 	{
           DEBUG_printf(("%s>>>> Auto-ascend <<<\n", indent));
 
-          if (temp && ch != '/' &&
+          if (ch != '/' &&
 	      temp->markup != MARKUP_BODY &&
 	      temp->markup != MARKUP_DD &&
 	      temp->markup != MARKUP_DT &&
@@ -610,7 +665,6 @@ htmlReadFile(tree_t     *parent,	// I - Parent tree entry
             indent[strlen((char *)indent) - 4] = '\0';
 #endif // DEBUG
 
-
           // Safety check; should never happen, since MARKUP_FILE is
 	  // the root node created by the caller...
           if (temp->parent)
@@ -730,7 +784,7 @@ htmlReadFile(tree_t     *parent,	// I - Parent tree entry
             if (ptr < (s + sizeof(s) - 1))
 	      *ptr++ = ';';
 	  }
-	  else if (ptr < (s + sizeof(s) - 1))
+	  else
 	    *ptr++ = ch;
         }
 	else if (ch != 0 && ch != '\r')
@@ -783,13 +837,14 @@ htmlReadFile(tree_t     *parent,	// I - Parent tree entry
 	    else
 	      *eptr++ = ch;
 
+          *eptr = '\0';
+
           if (ch != ';')
 	  {
 	    ungetc(ch, fp);
 	    ch = 0;
 	  }
 
-          *eptr = '\0';
           if (!ch)
 	  {
 	    progress_error(HD_ERROR_HTML_ERROR, "Unquoted & on line %d.",
@@ -812,7 +867,7 @@ htmlReadFile(tree_t     *parent,	// I - Parent tree entry
             if (ptr < (s + sizeof(s) - 1))
 	      *ptr++ = ';';
 	  }
-	  else if (ptr < (s + sizeof(s) - 1))
+	  else
 	    *ptr++ = ch;
         }
 	else if (ch)
@@ -835,7 +890,8 @@ htmlReadFile(tree_t     *parent,	// I - Parent tree entry
       t->markup = MARKUP_NONE;
       t->data   = (uchar *)strdup((char *)s);
 
-      DEBUG_printf(("%sfragment \"%s\", line %d\n", indent, s, linenum));
+      DEBUG_printf(("%sfragment \"%s\" (len=%d), line %d\n", indent, s,
+                    (int)(ptr - s), linenum));
     }
 
    /*
@@ -843,6 +899,9 @@ htmlReadFile(tree_t     *parent,	// I - Parent tree entry
     * entry we've read, set the child pointer...
     */
 
+    DEBUG_printf(("%sADDING %s node to %s parent!\n", indent,
+                  _htmlMarkups[t->markup],
+		  parent ? _htmlMarkups[parent->markup] : "ROOT"));
     if (parent != NULL && prev == NULL)
       parent->child = t;
 
@@ -975,7 +1034,8 @@ htmlReadFile(tree_t     *parent,	// I - Parent tree entry
           break;
 
       case MARKUP_PRE :
-          t->typeface      = TYPE_COURIER;
+          t->typeface      = _htmlBodyFont >= TYPE_MONOSPACE ? TYPE_MONOSPACE
+	                                                     : TYPE_COURIER;
           t->size          = SIZE_PRE;
           t->style         = STYLE_NORMAL;
           t->subscript     = 0;
@@ -1095,7 +1155,13 @@ htmlReadFile(tree_t     *parent,	// I - Parent tree entry
             for (ptr = face; *ptr != '\0'; ptr ++)
               *ptr = tolower(*ptr);
 
-            if (strstr((char *)face, "arial") != NULL ||
+            if (strstr((char *)face, "serif") != NULL)
+              t->typeface = TYPE_SERIF;
+            else if (strstr((char *)face, "sans") != NULL)
+              t->typeface = TYPE_SANS_SERIF;
+            else if (strstr((char *)face, "mono") != NULL)
+              t->typeface = TYPE_MONOSPACE;
+            else if (strstr((char *)face, "arial") != NULL ||
                 strstr((char *)face, "helvetica") != NULL)
               t->typeface = TYPE_HELVETICA;
             else if (strstr((char *)face, "times") != NULL)
@@ -1104,6 +1170,8 @@ htmlReadFile(tree_t     *parent,	// I - Parent tree entry
               t->typeface = TYPE_COURIER;
 	    else if (strstr((char *)face, "symbol") != NULL)
               t->typeface = TYPE_SYMBOL;
+	    else if (strstr((char *)face, "dingbat") != NULL)
+              t->typeface = TYPE_DINGBATS;
           }
 
           if ((color = htmlGetVariable(t, (uchar *)"COLOR")) != NULL)
@@ -1226,11 +1294,13 @@ htmlReadFile(tree_t     *parent,	// I - Parent tree entry
 	    have_whitespace = 0;
 	  }
 
-          t->typeface = TYPE_COURIER;
+          t->typeface = _htmlBodyFont >= TYPE_MONOSPACE ? TYPE_MONOSPACE
+	                                                : TYPE_COURIER;
 
           descend = 1;
           break;
 
+      case MARKUP_STRONG :
       case MARKUP_B :
           t->style = (style_t)(t->style | STYLE_BOLD);
 
@@ -1246,13 +1316,12 @@ htmlReadFile(tree_t     *parent,	// I - Parent tree entry
       case MARKUP_VAR :
           t->style = (style_t)(t->style | STYLE_ITALIC);
       case MARKUP_DFN :
-          t->typeface = TYPE_HELVETICA;
+          t->typeface = _htmlBodyFont >= TYPE_MONOSPACE ? TYPE_SANS_SERIF
+	                                                : TYPE_HELVETICA;
 
           descend = 1;
           break;
 
-      case MARKUP_STRONG :
-          t->style = (style_t)(t->style | STYLE_BOLD);
       case MARKUP_CITE :
       case MARKUP_EM :
       case MARKUP_I :
@@ -1726,7 +1795,8 @@ htmlNewTree(tree_t   *parent,	/* I - Parent entry */
         break;
 
     case MARKUP_PRE :
-        t->typeface      = TYPE_COURIER;
+        t->typeface      = _htmlBodyFont >= TYPE_MONOSPACE ? TYPE_MONOSPACE
+	                                                   : TYPE_COURIER;
         t->size          = SIZE_PRE;
         t->style         = STYLE_NORMAL;
         t->subscript     = 0;
@@ -1978,6 +2048,85 @@ htmlGetVariable(tree_t *t,	/* I - Tree entry */
 }
 
 
+/*
+ * 'htmlLoadFontWidths()' - Load all of the font width files.
+ */
+
+void
+htmlLoadFontWidths(void)
+{
+  int		i, j;			/* Looping vars */
+  char		filename[1024];		/* Filenames */
+  FILE		*fp;			/* Files */
+  int		ch;			/* Character */
+  float		width;			/* Width value */
+  char		glyph[64];		/* Glyph name */
+  char		line[1024];		/* Line from AFM file */
+
+
+ /*
+  * Now read all of the font widths...
+  */
+
+  for (i = 0; i < TYPE_MAX; i ++)
+    for (j = 0; j < STYLE_MAX; j ++)
+    {
+      for (ch = 0; ch < 256; ch ++)
+        _htmlWidths[i][j][ch] = 0.6f;
+
+      snprintf(filename, sizeof(filename), "%s/fonts/%s.afm", _htmlData,
+               _htmlFonts[i][j]);
+      if ((fp = fopen(filename, "r")) == NULL)
+      {
+#ifndef DEBUG
+        progress_error(HD_ERROR_FILE_NOT_FOUND,
+                       "Unable to open font width file %s!", filename);
+#endif /* !DEBUG */
+        continue;
+      }
+
+      while (fgets(line, sizeof(line), fp) != NULL)
+      {
+        if (strncmp(line, "C ", 2) != 0)
+	  continue;
+
+        if (i < TYPE_SYMBOL)
+	{
+	 /*
+	  * Handle encoding of Courier, Times, and Helvetica using
+	  * assigned charset...
+	  */
+
+          if (sscanf(line, "%*s%*s%*s%*s%f%*s%*s%s", &width, glyph) != 2)
+	    continue;
+
+          for (ch = 0; ch < 256; ch ++)
+	    if (_htmlGlyphs[ch] && !strcmp(_htmlGlyphs[ch], glyph))
+	      _htmlWidths[i][j][ch] = width * 0.001f;
+	}
+	else
+	{
+	 /*
+	  * Symbol and dingbats fonts uses their own encoding...
+	  */
+
+          if (sscanf(line, "%*s%d%*s%*s%f", &ch, &width) != 2)
+	    continue;
+
+          if (ch < 256 && ch >= 0)
+	    _htmlWidths[i][j][ch] = width * 0.001f;
+	}
+      }
+
+      fclose(fp);
+
+      // Make sure that non-breaking space has the same width as
+      // a breaking space...
+      _htmlWidths[i][j][160] = _htmlWidths[i][j][32];
+    }
+}
+
+
 /*
  * 'htmlSetVariable()' - Set a variable for a markup entry.
  */
@@ -2076,16 +2225,15 @@ htmlSetBaseSize(float p,	/* I - Point size of paragraph font */
  */
 
 void
-htmlSetCharSet(const char *cs)	/* I - Character set file to load */
+htmlSetCharSet(const char *cs)		/* I - Character set file to load */
 {
-  int		i, j;		/* Looping vars */
-  char		filename[1024];	/* Filenames */
-  FILE		*fp;		/* Files */
-  int		ch, unicode;	/* Character values */
-  float		width;		/* Width value */
-  char		glyph[64];	/* Glyph name */
-  char		line[1024];	/* Line from AFM file */
-  int		chars[256];	/* Character encoding array */
+  int		i, j;			/* Looping vars */
+  char		filename[1024];		/* Filenames */
+  FILE		*fp;			/* Files */
+  int		ch, unicode;		/* Character values */
+  char		glyph[64];		/* Glyph name */
+  char		line[1024];		/* Line from charset file */
+  int		chars[256];		/* Character encoding array */
 
 
   strlcpy(_htmlCharSet, cs, sizeof(_htmlCharSet));
@@ -2135,36 +2283,6 @@ htmlSetCharSet(const char *cs)	/* I - Character set file to load */
 
     for (i = 0; i < 256; i ++)
       chars[i] = i;
-
-   /*
-    * Hardcode characters 128 to 159 for Microsoft's version of ISO-8859-1...
-    */
-
-    chars[0x80] = 0x20ac; /* Euro */
-    chars[0x82] = 0x201a;
-    chars[0x83] = 0x0192;
-    chars[0x84] = 0x201e;
-    chars[0x85] = 0x2026;
-    chars[0x86] = 0x2020;
-    chars[0x87] = 0x2021;
-    chars[0x88] = 0x02c6;
-    chars[0x89] = 0x2030;
-    chars[0x8a] = 0x0160;
-    chars[0x8b] = 0x2039;
-    chars[0x8c] = 0x0152;
-    chars[0x91] = 0x2018;
-    chars[0x92] = 0x2019;
-    chars[0x93] = 0x201c;
-    chars[0x94] = 0x201d;
-    chars[0x95] = 0x2022;
-    chars[0x96] = 0x2013;
-    chars[0x97] = 0x2014;
-    chars[0x98] = 0x02dc;
-    chars[0x99] = 0x2122;
-    chars[0x9a] = 0x0161;
-    chars[0x9b] = 0x203a;
-    chars[0x9c] = 0x0153;
-    chars[0x9f] = 0x0178;
   }
   else
   {
@@ -2184,72 +2302,22 @@ htmlSetCharSet(const char *cs)	/* I - Character set file to load */
   * Build the glyph array...
   */
 
-  for (i = 0; i < 256; i ++)
+  for (i = 0, j = 0; i < 256; i ++)
+  {
+   /*
+    * Add the glyph to the charset array...
+    */
+
     if (chars[i] == 0)
+    {
       _htmlGlyphs[i] = NULL;
+      continue;
+    }
     else
       _htmlGlyphs[i] = _htmlGlyphsAll[chars[i]];
+  }
 
- /*
-  * Now read all of the font widths...
-  */
-
-  for (i = 0; i < 4; i ++)
-    for (j = 0; j < 4; j ++)
-    {
-      for (ch = 0; ch < 256; ch ++)
-        _htmlWidths[i][j][ch] = 0.6f;
-
-      snprintf(filename, sizeof(filename), "%s/fonts/%s.afm", _htmlData,
-               _htmlFonts[i][j]);
-      if ((fp = fopen(filename, "r")) == NULL)
-      {
-#ifndef DEBUG
-        progress_error(HD_ERROR_FILE_NOT_FOUND,
-                       "Unable to open font width file %s!", filename);
-#endif /* !DEBUG */
-        continue;
-      }
-
-      while (fgets(line, sizeof(line), fp) != NULL)
-      {
-        if (strncmp(line, "C ", 2) != 0)
-	  continue;
-
-        if (i < 3)
-	{
-	 /*
-	  * Handle encoding of Courier, Times, and Helvetica using
-	  * assigned charset...
-	  */
-
-          if (sscanf(line, "%*s%*s%*s%*s%f%*s%*s%s", &width, glyph) != 2)
-	    continue;
-
-          for (ch = 0; ch < 256; ch ++)
-	    if (_htmlGlyphs[ch] && strcmp(_htmlGlyphs[ch], glyph) == 0)
-	      _htmlWidths[i][j][ch] = width * 0.001f;
-	}
-	else
-	{
-	 /*
-	  * Symbol font uses its own encoding...
-	  */
-
-          if (sscanf(line, "%*s%d%*s%*s%f", &ch, &width) != 2)
-	    continue;
-
-          if (ch < 256 && ch >= 0)
-	    _htmlWidths[i][j][ch] = width * 0.001f;
-	}
-      }
-
-      fclose(fp);
-
-      // Make sure that non-breaking space has the same width as
-      // a breaking space...
-      _htmlWidths[i][j][160] = _htmlWidths[i][j][32];
-    }
+  htmlLoadFontWidths();
 }
 
 
@@ -2528,7 +2596,7 @@ parse_markup(tree_t *t,		/* I - Current tree entry */
             if (cptr < (comment + sizeof(comment) - 1))
 	      *cptr++ = ';';
 	  }
-	  else if (cptr < (comment + sizeof(comment) - 1))
+	  else
 	    *cptr++ = ch;
 	}
 	else
@@ -3052,6 +3120,8 @@ fix_filename(char *filename,		/* I - Original filename */
   static char	newfilename[1024];	/* New filename */
 
 
+//  printf("fix_filename(filename=\"%s\", base=\"%s\")\n", filename, base);
+
   if (filename == NULL)
     return (NULL);
 
@@ -3118,6 +3188,8 @@ fix_filename(char *filename,		/* I - Original filename */
 
   strlcat(newfilename, filename, sizeof(newfilename));
 
+//  printf("    newfilename=\"%s\"\n", newfilename);
+
   return (file_find(Path, newfilename));
 }
 
@@ -3297,5 +3369,5 @@ htmlFixLinks(tree_t *doc,		// I - Top node
 
 
 /*
- * End of "$Id: htmllib.cxx,v 1.41.2.79 2004/10/21 19:05:04 mike Exp $".
+ * End of "$Id: htmllib.cxx 1466 2005-11-18 01:37:50Z mike $".
  */
